unit WWWStartMain;

interface

uses
  Windows, SysUtils, Classes,
  Controls, Forms, RXShell, WinSvc,
  Menus, ExtCtrls, StdCtrls;

type
  TfrmMain = class(TForm)
    tmHide: TTimer;
    tmStart: TTimer;
    Tray: TRxTrayIcon;
    pmPopup: TPopupMenu;
    mmiRestore: TMenuItem;
    mmiHide: TMenuItem;
    btnQuit: TButton;
    lblService: TLabel;
    lblInterval: TLabel;
    Label1: TLabel;
    procedure tmHideTimer(Sender: TObject);
    procedure tmStartTimer(Sender: TObject);
    procedure mmiHideClick(Sender: TObject);
    procedure mmiRestoreClick(Sender: TObject);
    procedure btnQuitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
    Force : boolean;
    Interval : integer;
    Friendly,
    Service : string;
    RestartLog : TStringList;
    procedure RestartService;
    procedure ServiceControl(Control: DWORD);
    procedure GetDisplayName;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.tmHideTimer(Sender: TObject);
begin
  tmHide.Enabled := False;
  Hide;
end;

procedure TfrmMain.ServiceControl(Control: DWORD);
var
  SvcMgr		,
  Svc			: SC_HANDLE;
  ServiceStatus		: TServiceStatus;
begin
  SvcMgr := OpenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
  if SvcMgr = 0 then
    Begin
      RaiseLastWin32Error;
      RestartLog.Clear;
      RestartLog.LoadFromFile(Service+'_Restart.log');
      RestartLog.Add('Open SC Manager Error: '+DateTimeToStr(Now)+' '+SysErrorMessage(GetLastError));
      RestartLog.SaveToFile(Service+'_Restart.log');
    End;
  try
    Svc := OpenService(SvcMgr, PChar(Service), SERVICE_ALL_ACCESS);
    if Svc = 0 then
      Begin
        RaiseLastWin32Error;
        RestartLog.Clear;
        RestartLog.LoadFromFile(Service+'_Restart.log');
        RestartLog.Add('Open Service Error: '+DateTimeToStr(Now)+' '+SysErrorMessage(GetLastError));
        RestartLog.SaveToFile(Service+'_Restart.log');
      End;
    try
      ControlService(Svc, Control, ServiceStatus);
      RestartLog.Clear;
      RestartLog.LoadFromFile(Service+'_Restart.log');
      RestartLog.Add(DateTimeToStr(Now)+'; Service Restarter stopped the service '+Friendly);
      RestartLog.SaveToFile(Service+'_Restart.log');
    finally
      CloseServiceHandle(Svc);
    end;
  finally
    CloseServiceHandle(SvcMgr);
  end;
end;

procedure TfrmMain.tmStartTimer(Sender: TObject);
begin
  IF Force THEN
    Begin
      ServiceControl(SERVICE_CONTROL_STOP);
      Sleep(1000);
    End;
  RestartService;
end;

procedure TfrmMain.RestartService;
var
  SvcMgr,
  Svc			: SC_HANDLE;
  Nuffin : PChar;
  Buf : cardinal;
begin
  Nuffin := nil;
  SvcMgr := OpenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
  if SvcMgr = 0 then
    Begin
      RaiseLastWin32Error;
      RestartLog.Clear;
      RestartLog.LoadFromFile(Service+'_Restart.log');
      RestartLog.Add('Open SC Manager Error: '+DateTimeToStr(Now)+' '+SysErrorMessage(GetLastError));
      RestartLog.SaveToFile(Service+'_Restart.log');
    End;
  try
    Svc := OpenService(SvcMgr, PChar(Service), SERVICE_ALL_ACCESS);
    if Svc = 0 then
      Begin
        RaiseLastWin32Error;
        RestartLog.Clear;
        RestartLog.LoadFromFile(Service+'_Restart.log');
        RestartLog.Add('Open Service Error: '+DateTimeToStr(Now)+' '+SysErrorMessage(GetLastError));
        RestartLog.SaveToFile(Service+'_Restart.log');
      End;
    try
      IF StartService(Svc, 0, Nuffin) THEN
        Begin
          RestartLog.Clear;
          RestartLog.LoadFromFile(Service+'_Restart.log');
          RestartLog.Add(DateTimeToStr(Now)+'; Service Restarter started the service '+Friendly);
          RestartLog.SaveToFile(Service+'_Restart.log');
        End;
    finally
      CloseServiceHandle(Svc);
    end;
  finally
    CloseServiceHandle(SvcMgr);
  end;
End;

procedure TfrmMain.mmiHideClick(Sender: TObject);
begin
  Hide;
end;

procedure TfrmMain.mmiRestoreClick(Sender: TObject);
begin
  Show;
end;

procedure TfrmMain.btnQuitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Force := False;
  Interval := 0;

  Service := ParamStr(1);
  try
  Interval :=  StrToInt(ParamStr(2))*60000;
  except
  end;
  try
  IF UpperCase(ParamStr(3)) = 'FORCE' THEN
    Force := True;
  except
    Force := False;
  end;
  IF Service = '' THEN
    Service := 'W3SVC';

  IF Interval = 0 THEN
    Interval := 600000;
  tmStart.Interval := Interval;
  lblService.Caption := Service;
  lblInterval.Caption := 'Every '+IntToStr(Interval DIV 60000)+' minutes';
  Tray.Hint := Service+' Watchdog';
  RestartLog := TStringList.Create;
  RestartLog.Add('Program Started: '+DateTimeToStr(Now));
  RestartLog.SaveToFile(Service+'_Restart.log');
  tmStart.Enabled := True;
  tmHide.Enabled := True;
  GetDisplayName;

end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FreeAndNil(RestartLog);
end;

procedure TfrmMain.GetDisplayName;
VAR
  SvcMgr : SC_Handle;
  ServiceName : array[0..255] of char;
  Card : cardinal;
  WhatService : string;
Begin
  Card := SizeOf(ServiceName);
  WhatService := Service;//lblService.Caption;
  SvcMgr := OpenSCManager(nil,nil,SC_MANAGER_ALL_ACCESS);
  IF NOT GetServiceDisplayName(SvcMgr,PChar(WhatService),ServiceName,Card) THEN
    Begin
      MessageBox(0,PChar(SysErrorMessage(GetLastError)+' '+IntToStr(Card)),'Error',mb_OK);
    End;
  Friendly := '"'+StrPas(ServiceName)+'"';
End;

end.
